@@ -1,16 +1,23 @@
-from __future__ import absolute_import
-from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
-from .django_compat import get_app_names
+import importlib
+import os
 import sys
 import traceback
 
+from django.apps import apps
+from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
+
 __all__ = (
-    'import_settings_class',
-    'import_class',
-    'import_apps_submodule',
+    "import_settings_class",
+    "import_class",
+    "import_apps_submodule",
 )
 
+# Windows support
+IMPORT_PATH_IMPORTLIB = importlib.__file__.rstrip("c")  # .pyc -> .py
+IMPORT_PATH_GEVENT = os.path.join("gevent", "builtins.py")
+IMPORT_PATH_PYDEV = os.path.sep + "_pydev_"
+
 
 def import_settings_class(setting_name):
     """
@@ -18,7 +25,7 @@ def import_settings_class(setting_name):
     """
     config_value = getattr(settings, setting_name)
     if config_value is None:
-        raise ImproperlyConfigured("Required setting not found: {0}".format(setting_name))
+        raise ImproperlyConfigured(f"Required setting not found: {setting_name}")
 
     return import_class(config_value, setting_name)
 
@@ -27,10 +34,10 @@ def import_class(import_path, setting_name=None):
     """
     Import a class by name.
     """
-    mod_name, class_name = import_path.rsplit('.', 1)
+    mod_name, class_name = import_path.rsplit(".", 1)
 
     # import module
-    mod = _import_module(mod_name, classnames=(class_name,))
+    mod = import_module_or_none(mod_name)
     if mod is not None:
         # Loaded module, get attribute
         try:
@@ -40,32 +47,38 @@ def import_class(import_path, setting_name=None):
 
     # For ImportError and AttributeError, raise the same exception.
     if setting_name:
-        raise ImproperlyConfigured("{0} does not point to an existing class: {1}".format(setting_name, import_path))
+        raise ImproperlyConfigured(
+            f"{setting_name} does not point to an existing class: {import_path}"
+        )
     else:
-        raise ImproperlyConfigured("Class not found: {0}".format(import_path))
+        raise ImproperlyConfigured(f"Class not found: {import_path}")
 
 
 def import_apps_submodule(submodule):
     """
     Look for a submodule is a series of packages, e.g. ".pagetype_plugins" in all INSTALLED_APPS.
     """
-    apps = []
-    for app in get_app_names():
-        if _import_module('{0}.{1}'.format(app, submodule)) is not None:
-            apps.append(app)
+    found_apps = []
+    for appconfig in apps.get_app_configs():
+        app = appconfig.name
+        if import_module_or_none(f"{app}.{submodule}") is not None:
+            found_apps.append(app)
 
-    return apps
+    return found_apps
 
 
-# Based on code from django-oscar:
-def _import_module(module_label, classnames=()):
+def import_module_or_none(module_label):
     """
     Imports the module with the given name.
-    Returns None if the module doesn't exist, but propagates any import errors.
+
+    Returns None if the module doesn't exist,
+    but it does propagates import errors in deeper modules.
     """
     try:
-        return __import__(module_label, fromlist=classnames)
+        # On Python 3, importlib has much more functionality compared to Python 2.
+        return importlib.import_module(module_label)
     except ImportError:
+        # Based on code from django-oscar:
         # There are 2 reasons why there could be an ImportError:
         #
         #  1. Module does not exist. In that case, we ignore the import and return None
@@ -78,9 +91,14 @@ def _import_module(module_label, classnames=()):
         # application was found and ImportError originates within the local app
         __, __, exc_traceback = sys.exc_info()
         frames = traceback.extract_tb(exc_traceback)
+        frames = [
+            f
+            for f in frames
+            if f[0] != "<frozen importlib._bootstrap>"
+            and f[0] != IMPORT_PATH_IMPORTLIB  # Python 3.6
+            and not f[0].endswith(IMPORT_PATH_GEVENT)
+            and not IMPORT_PATH_PYDEV in f[0]
+        ]
         if len(frames) > 1:
-            if len(frames) == 2 and '/_pydev_' in frames[1][0] and frames[1][2] == 'plugin_import':
-                return None  # Ship over PyDev import hook, in PyCharm 4.x
-
             raise
     return None
