@@ -1,29 +1,33 @@
 """
 Optional integration with django-contrib-comments
 
-This avoids loading django.contrib.comments unless it's installed.
+This avoids loading django_comments or django.contrib.comments unless it's installed.
 All functions even work without having the app installed,
 and return stub or dummy values so all code works as expected.
 """
-import django
+from django.conf import settings
+from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.sites.models import Site
-from django.utils.translation import ugettext_lazy as _
 from django.db import models
-from fluent_utils.django_compat import is_installed, AUTH_USER_MODEL
-from fluent_utils.django_compat.moves.contenttypes import GenericForeignKey, GenericRelation
+from django.dispatch import Signal
+from django.utils.translation import gettext_lazy as _
+
+from fluent_utils.django_compat import is_installed
 
 __all__ = (
-    'django_comments',                # Main module
-    'get_model',                      # Get the comment model
-    'get_public_comments_for_model',  # Get publicly visible comments
-    'get_comments_are_open',          # Utility to check if comments are open for a model.
-    'get_comments_are_moderated',     # Utility to check if comments are moderated for a model.
-    'CommentModel',                   # Points to the comments model.
-    'CommentModerator',               # Base class for all custom comment moderators
-    'CommentsRelation',               # Generic relation back to the comments.
-    'CommentsMixin',                  # Model mixin for comments
-    'IS_INSTALLED',
+    "django_comments",  # Main module
+    "signals",  # Signals module
+    "get_model",  # Get the comment model
+    "get_form",  # Get the comment form
+    "get_public_comments_for_model",  # Get publicly visible comments
+    "get_comments_are_open",  # Utility to check if comments are open for a model.
+    "get_comments_are_moderated",  # Utility to check if comments are moderated for a model.
+    "CommentModel",  # Points to the comments model.
+    "CommentModerator",  # Base class for all custom comment moderators
+    "CommentsRelation",  # Generic relation back to the comments.
+    "CommentsMixin",  # Model mixin for comments
+    "IS_INSTALLED",
 )
 
 django_comments = None
@@ -32,22 +36,27 @@ CommentModerator = None
 get_model = None
 IS_INSTALLED = False
 
-if is_installed('django.contrib.comments'):
-    # Django 1.7 and below
-    from django.contrib import comments as django_comments
-    from django.contrib.comments import get_model
-    from django.contrib.comments.moderation import moderator, CommentModerator
-    IS_INSTALLED = True
-elif is_installed('django_comments'):
-    # as of Django 1.8, this is a separate app.
+if is_installed("django_comments"):
     import django_comments
-    from django_comments import get_model
-    from django_comments.moderation import moderator, CommentModerator
+    from django_comments import get_form, get_model, signals
+    from django_comments.moderation import CommentModerator, moderator
+
     IS_INSTALLED = True
 else:
+
     def get_model():
         return CommentManagerStub
 
+    def get_form():
+        raise NotImplementedError("No stub for comments.get_form() is implemented!")
+
+    class SignalsStub:
+        comment_will_be_posted = Signal()
+        comment_was_posted = Signal()
+        comment_was_flagged = Signal()
+
+    signals = SignalsStub()
+
 
 def get_public_comments_for_model(model):
     """
@@ -65,7 +74,7 @@ def get_comments_are_open(instance):
     """
     Check if comments are open for the instance
     """
-    if CommentModel is None:
+    if not IS_INSTALLED:
         return False
 
     try:
@@ -84,7 +93,7 @@ def get_comments_are_moderated(instance):
     """
     Check if comments are moderated for the instance
     """
-    if CommentModel is None:
+    if not IS_INSTALLED:
         return False
 
     try:
@@ -108,11 +117,7 @@ class CommentManagerStub(models.Manager):
     use_for_related_fields = True
 
     def get_queryset(self):
-        return super(CommentManagerStub, self).get_queryset().none()
-
-    if django.VERSION < (1,7):
-        def get_query_set(self):
-            return super(CommentManagerStub, self).get_query_set().none()
+        return super().get_queryset().none()
 
     def in_moderation(self):
         return self.none()
@@ -125,20 +130,26 @@ class CommentModelStub(models.Model):
     """
     Stub model that :func:`get_model` returns if *django.contrib.comments* is not installed.
     """
+
     class Meta:
         managed = False
-        app_label = 'django_comments'
+        app_label = "django_comments"
         db_table = "django_comments_stub"
 
     objects = CommentManagerStub()
 
     # add fields so ORM queries won't cause any issues.
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, on_delete=models.DO_NOTHING)
     object_pk = models.TextField()
     content_object = GenericForeignKey(ct_field="content_type", fk_field="object_pk")
-    site = models.ForeignKey(Site)
+    site = models.ForeignKey(Site, on_delete=models.CASCADE)
 
-    user = models.ForeignKey(AUTH_USER_MODEL, related_name="%(class)s_comments")
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL,
+        null=True,
+        related_name="%(class)s_comments",
+        on_delete=models.SET_NULL,
+    )
     user_name = models.CharField(max_length=50, blank=True)
     user_email = models.EmailField(blank=True)
     user_url = models.URLField(blank=True)
@@ -149,44 +160,49 @@ class CommentModelStub(models.Model):
     is_removed = models.BooleanField(default=False)
 
 
-
 CommentModel = get_model()
 
 
-
 if IS_INSTALLED:
-    class CommentRelation(GenericRelation):
+
+    class CommentsRelation(GenericRelation):
         def __init__(self, to=CommentModel, **kwargs):
-            kwargs.setdefault('object_id_field', 'object_pk')
-            super(CommentRelation, self).__init__(to, **kwargs)
+            kwargs.setdefault("object_id_field", "object_pk")
+            super().__init__(to, **kwargs)
+
+
 else:
-    class CommentRelation(models.Field):
+
+    class CommentsRelation(models.Field):
         def __init__(self, *args, **kwargs):
             pass
 
         def contribute_to_class(self, cls, name, virtual_only=False):
             setattr(cls, name, CommentModelStub.objects.none())
 
+CommentRelation = CommentsRelation  # noqa, was typo
 
 
 class CommentsMixin(models.Model):
     """
     Mixin for adding comments support to a model.
     """
+
     enable_comments = models.BooleanField(_("Enable comments"), default=True)
 
     # Reverse relation to the comments model.
     # This is a stub when django.contrib.comments is not installed, so templates don't break.
     # This avoids importing django.contrib.comments models when the app is not used.
-    all_comments = CommentRelation(verbose_name=_("Comments"))
+    all_comments = CommentsRelation(verbose_name=_("Comments"))
 
     class Meta:
         abstract = True
 
     # Properties
     comments = property(get_public_comments_for_model, doc="Return the visible comments.")
-    comments_are_moderated = property(get_comments_are_moderated, doc="Check if comments are moderated")
-
+    comments_are_moderated = property(
+        get_comments_are_moderated, doc="Check if comments are moderated"
+    )
 
     @property
     def comments_are_open(self):
@@ -197,5 +213,3 @@ class CommentsMixin(models.Model):
             return False
 
         return get_comments_are_open(self)
-
-
